from os.path import abspath
from typing import Any, Optional, Union

abspathu = abspath

def upath(path: Any): ...
def npath(path: Any): ...
def safe_join(base: Union[bytes, str], *paths: Any) -> str: ...
def symlinks_supported() -> Any: ...
